import subprocess
import os
import time
from charmhelpers.core.hookenv import (
    action_set,
    in_relation_hook,
    relation_get,
    relation_set,
    relation_ids
)

COLLECT_PROFILE_DATA = '/usr/local/bin/collect-profile-data'


class Benchmark():
    """
    Helper class for the `benchmark` interface.

    :param list actions: Define the actions that are also benchmarks


    Examples:

    Notify the Benchmark GUI of the actions that are benchmark-enabled,
    usually run from benchmark-relation-[joined|changed]::

        Benchmark(['memory', 'cpu', 'disk', 'smoke', 'custom'])


    From within a benchmark-enabled action::

        # Begins timing of a benchmark
        Benchmark.start()

        [... run benchmark and parse results ...]

        Benchmark.set_data({'results.transactions.value': 1096})
        Benchmark.set_data({'results.transactions.units': 'hits'})

        # Store a meta key, available via ``juju action fetch`` but not shown
        # in the Benchmark GUI
        Benchmark.set_meta('myuuid', '1b231f32-16c3-11e5-ac89-14109fd63717')

        # The higher the score, the better the benchmark
        Benchmark.set_composite_score(16.70, 'hits/sec', 'desc')

        # Finish the timing of a benchmark
        Benchmark.finish()
    """

    required_keys = [
        'hostname',
        'port',
        'graphite_port',
        'graphite_endpoint',
        'api_port'
    ]

    def __init__(self, benchmarks=None):
        if in_relation_hook():
            if benchmarks is not None:
                for rid in sorted(relation_ids('benchmark')):
                    relation_set(relation_id=rid, relation_settings={
                        'benchmarks': ",".join(benchmarks)
                    })

            # Check the relation data
            config = {}
            for key in self.required_keys:
                val = relation_get(key)
                if val is not None:
                    config[key] = val
                else:
                    # We don't have all of the required keys
                    config = {}
                    break

            if len(config):
                with open('/etc/benchmark.conf', 'w') as f:
                    for key, val in config.items():
                        f.write("%s=%s\n" % (key, val))

    @staticmethod
    def set_data(value):
        """
        Set the key:value to be passed to ``action_set``

        :param list value: A hash containing the key and value
        """
        try:
            action_set(value)
            return True
        except OSError:
            return False

    @staticmethod
    def set_composite_score(value, units='', direction=None):
        """
        Set the composite score for a benchmark run. This is a single number
        representative of the benchmark results. This could be the most
        important metric, or an amalgamation of metric scores.
        """
        Benchmark.set_data({'meta.composite.value': value})

        # The GUI expects this key, even if it's empty
        Benchmark.set_data({'meta.composite.units': units})

        if direction:
            Benchmark.set_data({'meta.composite.direction': direction})
        return True

    @staticmethod
    def set_meta(key, value):
        Benchmark.set_data({'meta.%s.value' % key: value})

    @staticmethod
    def start():
        """
        If the cabs-collector charm is installed, take a snapshot
        of the current profile data.
        """
        # Do profile data collection immediately on this unit
        if os.path.exists(COLLECT_PROFILE_DATA):
            subprocess.check_output([COLLECT_PROFILE_DATA])

        # Tell collector charm the action_id via the benchmark relation.
        # The collector will pass the action_id to its peers,
        # triggering profile data collection on all collectors in the
        # environment.
        for rid in relation_ids('benchmark'):
            relation_set(relation_id=rid, relation_settings={
                'action_id': os.environ.get('JUJU_ACTION_ID')
            })

        return Benchmark.set_data({
            'meta.start': time.strftime('%Y-%m-%dT%H:%M:%SZ')
        })

    @staticmethod
    def finish():
        return Benchmark.set_data({
            'meta.stop': time.strftime('%Y-%m-%dT%H:%M:%SZ')
        })



#!/usr/bin/env python
import argparse
from charms.benchmark import Benchmark


def main():
    parser = argparse.ArgumentParser(
        description='Set a meta key of the benchmark run.'
    )
    parser.add_argument(
        "key",
        metavar='key',
        help='The key of the data to store.'
    )

    parser.add_argument(
        "value",
        metavar='value',
        help='The value of the data to store.'
    )

    args = parser.parse_args()

    Benchmark.set_meta(args.key, args.value)

if __name__ == "__main__":
    main()



#!/usr/bin/env python
import argparse
from charms.benchmark import Benchmark


def main():
    parser = argparse.ArgumentParser(
        description='Inform the Benchmark GUI of available benchmarks'
    )
    parser.add_argument(
        "benchmarks",
        metavar='benchmark(s)',
        nargs='+',
        help='A space-delimited list of benchmarks exposed by the charm.'
    )
    args = parser.parse_args()

    Benchmark(args.benchmarks)


if __name__ == "__main__":
    main()



from testtools import TestCase

import mock
from charms.benchmark import Benchmark
from helpers import patch_open, FakeRelation


TO_PATCH = [
    'in_relation_hook',
    'relation_ids',
    'relation_set',
    'relation_get',
]

FAKE_RELATION = {
    'benchmark:0': {
        'benchmark/0': {
            'hostname': '127.0.0.1',
            'port': '1111',
            'graphite_port': '2222',
            'graphite_endpoint': 'http://localhost:3333',
            'api_port': '4444'
        }
    }
}


class TestBenchmark(TestCase):

    def setUp(self):
        super(TestBenchmark, self).setUp()
        for m in TO_PATCH:
            setattr(self, m, self._patch(m))
        self.fake_relation = FakeRelation(FAKE_RELATION)

        self.relation_get.side_effect = self.fake_relation.get
        self.relation_ids.side_effect = self.fake_relation.relation_ids

    def _patch(self, method):
        _m = mock.patch('charms.benchmark.' + method)
        m = _m.start()
        self.addCleanup(_m.stop)
        return m

    @mock.patch('charms.benchmark.action_set')
    def test_set_data(self, action_set):
        action_set.return_value = True
        data = {'key': 'value'}
        self.assertTrue(Benchmark.set_data(data))
        action_set.assert_called_once_with(data)

    @mock.patch('charms.benchmark.relation_get')
    @mock.patch('charms.benchmark.relation_set')
    @mock.patch('charms.benchmark.relation_ids')
    @mock.patch('charms.benchmark.in_relation_hook')
    def test_benchmark_init(self, in_relation_hook, relation_ids,
                            relation_set, relation_get):

        in_relation_hook.return_value = True
        relation_data = FAKE_RELATION['benchmark:0']['benchmark/0']
        relation_ids.return_value = FAKE_RELATION.keys()
        relation_get.side_effect = lambda k: relation_data.get(k)
        actions = ['asdf', 'foobar']

        with patch_open() as (_open, _file):
            b = Benchmark(actions)

            self.assertIsInstance(b, Benchmark)

            relation_ids.assert_called_once_with('benchmark')

            for key in b.required_keys:
                relation_get.assert_any_call(key)

            relation_set.assert_called_once_with(
                relation_id='benchmark:0',
                relation_settings={'benchmarks': ",".join(actions)}
            )

            # Test benchmark.conf
            _open.assert_called_with('/etc/benchmark.conf', 'w')
            for key, val in relation_data.items():
                _file.write.assert_any_call("%s=%s\n" % (key, val))

    @mock.patch('charms.benchmark.action_set')
    def test_benchmark_start_oserror(self, action_set):
        action_set.side_effect = OSError('File not found')
        self.assertFalse(Benchmark.start())

    @mock.patch('charms.benchmark.action_set')
    def test_benchmark_finish_oserror(self, action_set):
        action_set.side_effect = OSError('File not found')
        self.assertFalse(Benchmark.finish())

    @mock.patch.dict('charms.benchmark.os.environ', {
        'JUJU_ACTION_ID': 'my_action'})
    @mock.patch('charms.benchmark.relation_set')
    @mock.patch('charms.benchmark.relation_ids')
    @mock.patch('charms.benchmark.action_set')
    @mock.patch('os.path.exists')
    @mock.patch('subprocess.check_output')
    def test_benchmark_start(self, check_output, exists, action_set,
                             relation_ids, relation_set):

        exists.return_value = True
        check_output.return_value = "data"
        action_set.return_value = True
        relation_ids.return_value = ['benchmark:1']

        self.assertTrue(Benchmark.start())

        relation_set.assert_called_once_with(
            relation_id='benchmark:1',
            relation_settings={'action_id': 'my_action'}
        )

        COLLECT_PROFILE_DATA = '/usr/local/bin/collect-profile-data'
        exists.assert_any_call(COLLECT_PROFILE_DATA)
        check_output.assert_any_call([COLLECT_PROFILE_DATA])

    @mock.patch('charms.benchmark.action_set')
    def test_benchmark_finish(self, action_set):
        action_set.return_value = True
        self.assertTrue(Benchmark.finish())

    @mock.patch('charms.benchmark.Benchmark.set_data')
    def test_benchmark_set_composite_score(self, set_data):
        set_data.return_value = True
        self.assertTrue(Benchmark.set_composite_score(
            15.7, 'hits/sec', 'desc'))

    @mock.patch('charms.benchmark.Benchmark.set_data')
    @mock.patch('charms.benchmark.Benchmark.set_meta')
    def test_benchmark_meta(self, set_meta, set_data):
        key = 'foo'
        value = 'bar'
        units = 'bogomips'
        direction = 'desc'

        # Test with only a key/value pair
        Benchmark.set_meta(key, value)
        set_meta.assert_called_once_with(key, value)

        # set_data.side_effect = [True]
        # set_data.assert_has_calls([
        #     mock.call({'meta.%s' % key: value})
        #     #set_data({'meta.asdf': value})
        # ])
        # Benchmark.set_data({'meta.%s.value' % key: value})
        # Benchmark.set_data({'meta.%s.units' % key: units})
        # Benchmark.set_data({'meta.%s.direction' % key: direction})
        # set_data.reset_mock()
        set_meta.reset_mock()

        # Test with all parameters
        Benchmark.set_meta(key, value, units, direction)
        set_meta.assert_called_once_with(key, value, units, direction)
        pass
